# -*- coding: utf-8 -*-
"""
Created on Mon Nov 20 11:08:29 2017

@author: anna_work
"""
#%%

import matplotlib.pyplot as plt
import numpy as np
from sklearn.datasets import make_blobs
from sklearn.cluster import KMeans

#%%
sample_data, y = make_blobs(n_samples=150, 
                  n_features=2, 
                  centers=2, 
                  cluster_std=0.5, 
                  shuffle=True, 
                  random_state=0) # keep random_state = 0 so plots always come out the same


#%%

# use really crappy initial guess to illustrate how the algorithm converges
init_centers = np.array([
        [0., 2.5],
        [0, 2.5]
        ])



#%%
#### 1 iteration   ~Slide 20
# Run **1 iteration** of kmeans to see the initial (random) centroid location
# http://scikit-learn.org/stable/modules/generated/sklearn.cluster.KMeans.html
km1 = KMeans(n_clusters=2,
            init=init_centers, 
            n_init=1, 
            max_iter=1,
            precompute_distances =False,
            tol=1e-4,
            verbose=True,
            random_state=0)

y_km1 = km1.fit_predict(sample_data)

#%%

#### plot centroids of 1 iteration   ~Slide 21
# don't show the membership yet (eg plot in black and white)
# visualize the clusters 1 & 2, **as they are predicted by y_km**
km1_cluster_0 = sample_data[y_km1 == 0, :] # <== the data points that were assigned to cluster 0
km1_cluster_1 = sample_data[y_km1 == 1, :] # <== the data points that were assigned to cluster 1

# plot the 0 cluster
plt.scatter(km1_cluster_0[:, 0],
            km1_cluster_0[:, 1],
            s=50,
            c='white',
            marker='o',
            edgecolors= 'black',
            label='cluster 1')

# plot the 1 cluster
plt.scatter(km1_cluster_1[:, 0],
            km1_cluster_1[:, 1],
            s=50,
            c='white',
            marker='o',
            edgecolors= 'black',
            label='cluster 2')

# plot the centroids as large dots
plt.scatter(km1.cluster_centers_[:, 0],
            km1.cluster_centers_[:, 1],
            s=250, # make the size larger for these 3
            marker='o',
            c=['blue','red'],
            label='centroids')
plt.grid()
plt.title("Sample Clusters")
plt.xlabel("Variable 1")
plt.ylabel("Variable 2")

plt.savefig('dev/kmeans_1it_bw.png', dpi=200)
plt.close()

#%%
### plot cluster membership of 1 iteration

# plot the 0 cluster
plt.scatter(km1_cluster_0[:, 0],
            km1_cluster_0[:, 1],
            s=50,
            c='white',
            marker='o',
            edgecolors= 'blue',
            label='cluster 1')

# plot the 1 cluster
plt.scatter(km1_cluster_1[:, 0],
            km1_cluster_1[:, 1],
            s=50,
            c='white',
            marker='o',
            edgecolors= 'red',
            label='cluster 2')

# plot the centroids as large dots
plt.scatter(km1.cluster_centers_[:, 0],
            km1.cluster_centers_[:, 1],
            s=250, # make the size larger for these 3
            marker='o',
            c=['blue','red'],
            label='centroids')
plt.grid()
plt.title("Sample Clusters")
plt.xlabel("Variable 1")
plt.ylabel("Variable 2")

plt.savefig('dev/kmeans_1it.png', dpi=200)
plt.close()

#%%
### run 2 iterations 

km2 = KMeans(n_clusters=2,
            init=init_centers, 
            n_init=1, 
            max_iter=2,
            precompute_distances =False,
            tol=1e-4,
            verbose=True,
            random_state=0)

y_km2 = km2.fit_predict(sample_data)

#%%
### plot old centroid and new centroid

# plot the 0 cluster
plt.scatter(km1_cluster_0[:, 0],
            km1_cluster_0[:, 1],
            s=50,
            c='white',
            marker='o',
            edgecolors= 'blue',
            label='cluster 1')

# plot the 1 cluster
plt.scatter(km1_cluster_1[:, 0],
            km1_cluster_1[:, 1],
            s=50,
            c='white',
            marker='o',
            edgecolors= 'red',
            label='cluster 2')

# plot the centroids as large dots
plt.scatter(km1.cluster_centers_[:, 0],
            km1.cluster_centers_[:, 1],
            s=250, # make the size larger for these 3
            marker='o',
            c=['#ccccff','#ffb3b3'], # (light blue and light red)
            label='centroids')

# plot the centroids as large dots
plt.scatter(km2.cluster_centers_[:, 0],
            km2.cluster_centers_[:, 1],
            s=250, # make the size larger for these 3
            marker='o',
            c=['blue','red'],
            label='centroids')
plt.grid()
plt.title("Sample Clusters")
plt.xlabel("Variable 1")
plt.ylabel("Variable 2")


plt.savefig('dev/kmeans_1-2.png', dpi=200)
plt.close()

#%%
### plot new cluster membership
km2_cluster_0 = sample_data[y_km2 == 0, :] # <== the data points that were assigned to cluster 0
km2_cluster_1 = sample_data[y_km2 == 1, :] # <== the data points that were assigned to cluster 1


# plot the 0 cluster
plt.scatter(km2_cluster_0[:, 0],
            km2_cluster_0[:, 1],
            s=50,
            c='white',
            marker='o',
            edgecolors= 'blue',
            label='cluster 1')

# plot the 1 cluster
plt.scatter(km2_cluster_1[:, 0],
            km2_cluster_1[:, 1],
            s=50,
            c='white',
            marker='o',
            edgecolors= 'red',
            label='cluster 2')

# plot the centroids as large dots
plt.scatter(km2.cluster_centers_[:, 0],
            km2.cluster_centers_[:, 1],
            s=250, # make the size larger for these 3
            marker='o',
            c=['blue','red'],
            label='centroids')
plt.grid()
plt.title("Sample Clusters")
plt.xlabel("Variable 1")
plt.ylabel("Variable 2")

#%%
# this is the same plot that's created in Clustering_v2.py
# plt.savefig('plots/kmeans_2.png', dpi=200)
# plt.close()


