"""
1a. Generate four fake data sets using the make_blobs function. 
For each one use 150 samples, 2 features and 4 centers, but vary the cluster
standard deviation (cluster_std):
    Name the first data set ds1 and give it cluster_std of 0.2
    Name the second data set ds2 and give it cluster_sd of 0.5
    Name the third data set ds3 and give it cluster_sd of 0.8
    Name the fourth data set ds4 and give it cluster_sd of 1.1
 
The documentation for the make_blobs function is here:
  http://scikit-learn.org/stable/modules/generated/sklearn.datasets.make_blobs.html
"""


from sklearn.datasets import make_blobs
ds1, _ = make_blobs(n_samples=150,                   
						n_features=2,                   
						centers=3,                   
						cluster_std=0.2,                   
						shuffle=True,                   
						random_state=0)
	
ds2, _ = make_blobs(n_samples=150,                   
						n_features=2,                   
						centers=3,                   
						cluster_std=0.5,                   
						shuffle=True,                   
						random_state=0)

ds3, _ = make_blobs(n_samples=150,                   
						n_features=2,                   
						centers=3,                   
						cluster_std=0.8,                   
						shuffle=True,                   
						random_state=0)
	
ds4, _ = make_blobs(n_samples=150,                   
						n_features=2,                   
						centers=3,                   
						cluster_std=1.1,                   
						shuffle=True,                   
						random_state=0)
	


#%%
"""
1b. Check the shape of ds2 and view the first 10 rows.

This is some fake data that has 150 observations of two variable ("features"). 
In a real problem, your features would be whatever variables you care about
for your data.
"""

ds2.shape
ds2[1:10,:]


#%%
"""
1c. Make three lists:
    1 - a list called data_sets that contains the four data sets you made above.
    2 - a list called true_groups that contains the corresponding y values
    3 - a list called sds which has the corresponding standard deviation for each data set
"""

data_sets = [ds1, ds2, ds3, ds4]
sds = [0.2, 0.6, 0.8, 1.1]

#%%


"""
2. Plot each of the example data sets with the matplotlib scatter function.
You can make separate plots for each one, or put them all into a single plot
with multiple subplots. (For the second option, see your week 2 lecture 1 notes
on matplotlib!). 
Regardless of the plotting style you choose, make sure you label your x and y
axes, and that each plot has a title that reflects that particular data set's
standard deviation. 

Note: you can easily loop over the four data sets since you've put them into
a list. Therefore you don't need to write the plotting commands four separate
times. 
"""
import matplotlib.pyplot as plt


fig, axes = plt.subplots(nrows=2, ncols=2)

fig.suptitle("Sample data sets with varying standard deviation")
n = 0
m = 0
for i in range(4):
    data_set = data_sets[i]
    sd = sds[i]
    axes[n][m].scatter(
            data_set[:, 0], data_set[:, 1])
    axes[n][m].set_xlabel("Variable 1")
    axes[n][m].set_ylabel("Variable 2")
    axes[n][m].set_title("cluster_std = {}".format(sd))
    if m==1:
        m = 0
        n = n + 1
    else:
        m = m + 1
  
fig.tight_layout()
fig.set_size_inches(8,8)
fig.savefig("sample_data.png")

#%%

"""
3. Create an empty list called km_inertias. 
Then run KMeans on each of the four data sets (one by one or using a loop).
After each time you run it, append the km.inertia value to the km_inertias list.
Print out the km_inertias list. What do you notice? 
"""
from sklearn.cluster import KMeans
km = KMeans(n_clusters=3,             
				init='random',             
				n_init=10,             
				max_iter=300,            
				tol=1e-04,            
				random_state=0)

km_inertias = []
for data_set in data_sets:
    y_km = km.fit_predict(data_set)
    km_inertias.append(km.inertia_)

print(km_inertias)

"""Conclusion: As the standard deviation of our data increases, so does the inertia.
This makes sense because both values measure the tightness of the clusters."""
#%%
"""
4. Update your code from #3 to produce a plot of the clustering results.
 Each data point should be colored by the cluster it was assigned to.
"""


fig, axes = plt.subplots(nrows=2, ncols=2)
fig.suptitle("Results of K-Means clustering on sample data")  
n = 0
m = 0
for i in range(4):
    data_set = data_sets[i]
    sd = sds[i]
    y_km = km.fit_predict(data_set)
    axes[n][m].scatter(
            data_set[:, 0], data_set[:, 1],
            c=y_km)  # <-- color by the cluster number
    axes[n][m].set_xlabel("Variable 1")
    axes[n][m].set_ylabel("Variable 2")
    axes[n][m].set_title("cluster_std = {}".format(sd))
    if m==1:
        m = 0
        n = n + 1
    else:
        m = m + 1
  

fig.tight_layout()
fig.set_size_inches(8,8)
fig.savefig("sample_data-clustered.png")

#%%
"""
5. What does the fourth plot demonstrate about blindly using KMeans on any data set?
"""

"""
The original data for the cluster_std=1.1 data set really didn't have a meaningful
structure. KMeans will **always** create clusters on the data you give it,
so make sure there is truly a clustered structure for it to find.
"""