import matplotlib.pyplot as plt
import pandas as pd
from matplotlib import cm

from sklearn.cluster import KMeans

"""
1. Read in the small_candy csv file into a dataframe called candies
"""
candies = pd.read_csv('small_candy.csv')


#%%

"""
2. Set the Names field as the dataframe index
"""
candies = candies.set_index('Names')
# this leaves us with only the numerical data (the various flavor scores)


#%%
"""
3. Use the describe() dataframe method to examine the columns' values.

How many features are there and what are they? 
Notice they are all between 0 and 1 (look at min and max).
"""
candies.describe()

"""There are four features: Sweetness, Sourness, Nuttiness and Texture"""

#%%
"""
4. Run kMeans but **only on the Sweetness and Sourness features**.
(In lecture we ran Kmeans using all four features)
"""
# Step 3: Run kMeans
km3 = KMeans(n_clusters=3, 
            init='k-means++', 
            n_init=10, 
            max_iter=300,
            tol=1e-04,
            random_state=0)

# only use sweetness and sourness for the clustering
y_km3 = km3.fit_predict(candies[['Sweetness', 'Sourness']])


#%%
"""
5a. Plot sweetness vs sourness
The point should be colored by the cluster they were
assigned to. Be sure to label the x and y axes and
provide a title.

How does this plot compare to the one we made in lecture?
What explains this difference?
"""


plt.scatter(candies['Sweetness'], candies['Sourness'],
            s=50,
            c=y_km3)

plt.grid()
plt.xlabel('Sweetness')
plt.ylabel('Sourness')
plt.title("Candies Clusters")
plt.tight_layout()

"""
This clustering looks perfect!
 That's because we clustered on only these two variables, 
  so it better look perfect!
"""

plt.savefig('exercises/exercise2/sweet_sour.png', dpi=200)
# plt.close()
#%%
"""
5b. Plot sweetness vs nuttiness
"""
plt.scatter(candies['Sweetness'], candies['Nuttiness'],
            s=50,
            c=y_km3)

plt.grid()
plt.xlabel('Sweetness')
plt.ylabel('Nuttiness')
plt.title("Candies Clusters")
plt.tight_layout()

plt.savefig('exercises/exercise2/sweet_nutty.png', dpi=200)
plt.close()

#%%
"""
5c. Plot sweetness vs texture
"""

plt.scatter(candies['Sweetness'], candies['Texture'],
            s=50,
            c=y_km3)

plt.grid()
plt.xlabel('Sweetness')
plt.ylabel('Texture')
plt.title("Candies Clusters")
plt.tight_layout()

plt.savefig('exercises/exercise2/sweet_texture.png', dpi=200)
plt.close()


#%%
"""
5d. Plot nuttiness vs texture
"""
plt.scatter(candies['Nuttiness'], candies['Texture'],
            s=50,
            c=y_km3)

plt.grid()
plt.xlabel('Nuttiness')
plt.ylabel('Texture')
plt.title("Candies Clusters")
plt.tight_layout()

plt.savefig('exercises/exercise2/nutty_texture.png', dpi=200)
plt.close()

#%%