"""
1. Read in the animals.csv file. The variables are name,
which gives an abbreviation of the animal name. The other
columns have the following meaning:
    
war	warm-blooded
fly	can fly
ver	vertebrate
end	endangered
gro	live in groups
hai	have hair

A value of 1 indicates "Yes" and a value of 0 indicates "No."

How many animals are in the data set and what are they?
"""
import pandas as pd
animals = pd.read_csv("animals.csv")

animals.columns

animals.shape  # 20 animals
animals['name'] # names of the animals
#%%
"""
2a. Move the name column to the dataframe index
"""
animals.set_index("name", inplace=True)

#%%
"""
2b. Use the .loc method to pull the information about cows (cow)
and lizards (liz)
"""
animals.loc['cow']
animals.loc['liz']

# or
animals.loc[['cow', 'liz']]


#%%
"""
2c. Calculate the difference between cow and lizard
(Refer to slide 21 for the distance formula)
"""
from math import sqrt
sqrt( (1-0)**2 + (0-0)**2 + (1-1)**2 + (0-0)**2 + (1-0)**2 + (1-0)**2)
# = 1.7320

# or 
sqrt(sum( (animals.loc['cow'] - animals.loc['liz']).pow(2) ) )
# = 1.7320

#%%
"""
3a. Compute the distance matrix between all of the animals
"""
from scipy.spatial.distance import pdist
dist = pdist(animals, metric='euclidean')

#%%
"""
3b. Use the squareform function to convert the dist array
you made in the previous step into a square matrix. 
Name the result dist_mat. What are the dimensions of dist_mat?
Based on these dimensions, what does dist_mat tell you?
"""
from scipy.spatial.distance import squareform
dist_mat = squareform(dist)
dist_mat.shape

"""The dist_mat matrix is 20 by 20. So it is telling us the "distance"
between each pair of animals. For example, the first row is the distance
between ants and all the other animals.
"""

#%%
"""
3c. Open dist_mat in the variable explorer. 
Why do 0's appear on the diagonal elements of dist_mat?
What is the lowest value in the matrix other than 0? 
Pick a matrix entry that has this low value, and determine it's
row and column number (remember to start counting from 0)
"""

"""
The lowest value other than 0 is 1.412. This value occurs in several
places throughout the matrix. I'll use the value in the 0th row
and 1st column
"""
dist_mat[0][1]  # 1.414...

#%%
"""
3d. Which animals are these?
"""
animals.index[0] # ant
animals.index[1] # bee

#%%
"""
3e. Therefore, what does it mean to have a small distance value between
two animals?
"""

"""
Low distance means the two animals are more similar
"""

#%%
"""
4a. Turn dist_mat into a dataframe, with index and columns both
equal to the animal names. Name this dist_df
"""
dist_df = pd.DataFrame(dist_mat, 
                       columns=animals.index,
                       index=animals.index)


#%%
"""
4b. In step 2c you calculated the distance between cow and lizard.
View the distance_df and confirm the corresponding value is the same.
"""
"""
The (cow, liz) row in dist_df is 1.7320 which is the same value we found
by hand in step 2b.
"""
#%%
"""
4c. Find examples of animals that are similar and examples of 
animals that are different
"""

"""
Similar: 
     - ants and flys
     - whales and manatees
     
Different:
    - elephant and bee
    - rabbit and eagle

"""
#%%
"""
5. Use the linkage function to calculate hierarchical clusters
of the animals.
The arguments to linkage should be the dist array, and method="complete"
"""
from scipy.cluster.hierarchy import linkage
row_clusters = linkage(dist, method='complete')


#%%
"""
6. Plot a dendrogram of the clusters.
User labels=animals.index to label the dendrogram with the animal names
"""
import matplotlib.pyplot as plt
from scipy.cluster.hierarchy import dendrogram

dendrogram(row_clusters, labels=animals.index)

plt.title("Animal Hierarchical Clustering")
plt.tight_layout()

plt.savefig("animal_clusters.png", dpi=200)
#%%
"""
7. Take a moment to understand the plot. How are animals grouped?
Do these groupings make sense?
"""

"""
The groupings make sense -- we see the bugs are grouped together,
the mammals are grouped together, etc.
The neat thing is, we didn't specify anything about their true classification.
The algorithm got it right though, just based on the 6 features we had for
each (wam blooded, vertebrate, has hair, etc)
"""